/* -*- C++ -*-
    This file is part of ThreadWeaver.

    SPDX-FileCopyrightText: 2005-2013 Mirko Boehm <mirko@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "queuesignals_p.h"

ThreadWeaver::Private::QueueSignals_Private::QueueSignals_Private()
{
}

ThreadWeaver::Private::QueueSignals_Private::~QueueSignals_Private()
{
}
